package SliMP3::Pages;

# SliMP3 Server Copyright (C) 2001 Sean Adams, Slim Devices Inc.
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License, 
# version 2.

use strict;
use File::Spec::Functions qw(:ALL);
use POSIX;
use SliMP3::Misc;

use SliMP3::Strings qw(string);

my($NEWLINE) = "\012";

sub home {
	my($client, %params) = @_;
	my $output = "";
	my %listform;

	if (defined($params{'forget'})) {
		SliMP3::Client::forgetClient($params{'forget'});
	}
	if ($::nosetup) {
		$params{'nosetup'} = 1;
	}
	# fill out the client setup choices
	foreach my $player (SliMP3::Client::clients()) {
		next if (!SliMP3::Client::isSliMP3($player));
		$listform{'playername'} = SliMP3::Client::name($player);
		$listform{'playerid'} = $player->id;
		$listform{'player'} = $params{'player'};
		$listform{'skinOverride'} = $params{'skinOverride'};
		$params{'player_list'} .= &SliMP3::HTTP::filltemplatefile("homeplayer_list.html", \%listform);;
	}

	if (SliMP3::iTunes::useiTunesLibrary()) {
		$params{'nofolder'} = 1;
	}
	
	$output .= &SliMP3::HTTP::filltemplatefile("home.html", \%params);
	return $output;
}

sub browser {
	my($client, $httpclientsock, $output, %params) = @_;

	my $dir = defined($params{'dir'}) ? $params{'dir'} : "";

	my $item;
	my $itempath;
	my @items;
	my %list_form;
	my $current_player = "";
	my $playlist;
	my $fulldir = SliMP3::Misc::virtualToAbsolute($dir);

	$::d_http && msg("browse virtual path: " . $dir . "\n");
	$::d_http && msg("with absolute path: " . $fulldir . "\n");

	if (defined($client)) {
		$current_player = $client->id;;
	}

	if ($dir =~ /^__playlists/) {
		$playlist = 1;
		$params{'playlist'} = 1;

		if (!defined(SliMP3::Prefs::get("playlistdir") && !SliMP3::iTunes::useiTunesLibrary())) {
			$::d_http && msg("no valid playlists directory!!\n");
			return &SliMP3::HTTP::filltemplatefile("badpath.html", \%params);
		}

		if ($dir =~ /__current.m3u$/) {
			# write out the current playlist to a file with the special name __current
			if (defined($client)) {
				my $count = SliMP3::Playlist::count($client);
				$::d_http && msg("Saving playlist of $count items to $fulldir\n");
				if ($count) {
					SliMP3::Parse::writeM3U(\@{SliMP3::Playlist::playList($client)},$fulldir);
				}
				SliMP3::Info::updateCache($fulldir);
			} else {
				$::d_http && msg("no client, so we can't save a file!!\n");
				return &SliMP3::HTTP::filltemplatefile("badpath.html", \%params);
			}
		}

	} else {
		if (!defined(SliMP3::Prefs::get("mp3dir"))) {
			$::d_http && msg("no mp3dir, so we can't save a file!!\n");
			return &SliMP3::HTTP::filltemplatefile("badpath.html", \%params);;
		}
	}

	if (!$fulldir || !SliMP3::Info::isList($fulldir)) {
		# check if we're just showing itunes playlists
		if (SliMP3::iTunes::useiTunesLibrary()) {
			browser_addtolist_done($current_player, $httpclientsock, \%params, [], $output);
			return;
		} else {
			$::d_http && msg("the selected playlist $fulldir isn't good!!.\n");
			return &SliMP3::HTTP::filltemplatefile("badpath.html", \%params);
		}
	}

	# if they are renaming the playlist, let 'em.
	if ($playlist && $params{'newname'}) {
		my $newname = $params{'newname'};
		$newname =~ tr/A-Za-z0-9/ /cs;
		$newname .= ".m3u";
		if ($newname ne "") {
			my @newpath  = splitdir($fulldir);
			pop @newpath;
			my $container = catdir(@newpath);
			push @newpath, $newname;
			my $newfullname = catdir(@newpath);

			$::d_http && msg("renaming $fulldir to $newfullname\n");
			if ($newfullname ne $fulldir && !-e $newfullname && rename $fulldir, $newfullname) {
				SliMP3::Info::clearCache($container);
				SliMP3::Info::clearCache($fulldir);
				$fulldir = $newfullname;

				$dir = SliMP3::Misc::descendVirtual(SliMP3::Misc::ascendVirtual($dir), $newname);

				$params{'dir'} = $dir;
				$::d_http && msg("new dir value: $dir\n");
			} else {
				$::d_http && msg("Rename failed!\n");
				$params{'RENAME_WARNING'} = 1;
			}
		}
	} elsif ($playlist && $params{'delete'}) {
		if (-f $fulldir && unlink $fulldir) {
			my @newpath  = splitdir($fulldir);
			pop @newpath;
			my $container = catdir(@newpath);
			SliMP3::Info::clearCache($container);
			SliMP3::Info::clearCache($fulldir);
			$dir = SliMP3::Misc::ascendVirtual($dir);
			$params{'dir'} = $dir;
			$fulldir = SliMP3::Misc::virtualToAbsolute($dir);
		}
	}

	#
	# Make separate links for each component of the pwd
	#
	%list_form=();
	$list_form{'player'} = $current_player;
	$list_form{'myClientState'} = $client;
	$list_form{'skinOverride'} = $params{'skinOverride'};
	my $aggregate = "";
	if ($playlist) { $aggregate = "__playlists" };
	$params{'pwd_list'} = " ";
	my $lastpath;
	foreach my $c (splitdir($dir)) {
	    if ($c ne "" && $c ne "__playlists" && $c ne "__current.m3u") {
		    $aggregate = catdir($aggregate, $c);        # incrementally build the path for each link
		    $list_form{'dir'}=SliMP3::HTTP::escape($aggregate);
		    $c =~ /(.*)\.m3u$/;
		    if ($1) { $c = $1; }
		    
		    if (SliMP3::Info::isURL($c)) {
		    	$list_form{'shortdir'}= SliMP3::Info::standardTitle(undef, $c);
		    } else {
		    	$list_form{'shortdir'}=$c; #possibly make this into the TITLE of the playlist if this is a number
		    }
		    $params{'pwd_list'} .= &SliMP3::HTTP::filltemplatefile("browse_pwdlist.html", \%list_form);
		}
		if ($c) { $lastpath = $c; }
	}

	if (SliMP3::Info::isM3U($fulldir)) {
		$::d_http && msg("lastpath equals $lastpath\n");
		if ($lastpath eq "__current.m3u") {
			$params{'playlistname'} = string("UNTITLED");
			$params{'savebuttonname'} = string("SAVE");
		} else {
			$lastpath =~ s/(.*)\.m3u$/$1/;
			$params{'playlistname'} = $lastpath;
			$params{'savebuttonname'} = string("RENAME");
			$params{'titled'} = 1;
		}
	}

	my $items = [];

	SliMP3::Scan::addToList($items, $fulldir, 0, undef,  
			\&browser_addtolist_done, $current_player, $httpclientsock, \%params, $items, $output);  
	
	# when this finishes, it calls the next function, browser_addtolist_done:
	return ''; # return '' means we'll take care of sending the output to the client (special case because we're going into the background)
}

sub browser_addtolist_done {
	my (	$current_player,
		$httpclientsock,
	 	$paramsref,
		$itemsref,
		$output,
		$numitems
	) = @_;
	
	if (defined $paramsref->{'dir'} && $paramsref->{'dir'} eq '__playlists' && SliMP3::iTunes::useiTunesLibrary()) {
		$::d_http && msg("just showing itunes playlists\n");
		push @$itemsref, @{SliMP3::iTunes::playlists()};
		if (SliMP3::iTunes::stillScanning()) {
			$paramsref->{'warn'} = 1;
		}
	} 
	
	$numitems = scalar @{ $itemsref };
	
	$::d_http && msg("browser_addtolist_done with $numitems items (". scalar @{ $itemsref } . ")\n");

	my $shortitem;
	${$paramsref}{'browse_list'} = " ";
	if ($numitems) {
		my %list_form;
		my $item;
		my $otherparams = 	'player=' . SliMP3::HTTP::escape($current_player) .
							'&dir=' . SliMP3::HTTP::escape($paramsref->{'dir'}) . '&';
							
		my @namearray;
		foreach $item ( @{$itemsref} ) {
			$::d_http && msg("browser_addtolist_done getting name for $item\n");
			if ($item) {
				push @namearray, SliMP3::Info::standardTitle(undef,$item);
			}
		}
		my ($start,$end);
		
		($start,$end) = pagebar($numitems,
								$$paramsref{'path'},
								0,
								$otherparams,
								\$$paramsref{'start'},
								\$$paramsref{'browselist_header'},
								\$$paramsref{'browselist_pagebar'},
								$$paramsref{'skinOverride'});
		
		my $itemnumber = $start;
		my $offset = $start % 2 ? 0 : 1;
		my $lastAnchor = '';
		my $filesort = SliMP3::Prefs::get('filesort');
		foreach $item ( @{$itemsref}[$start..$end] ) {
			# don't display and old unsaved playlist
			if ($item =~ /__current.m3u$/) { next; }
			%list_form=();
			#
			# There are different templates for directories and playlist items:
			#
			$shortitem = SliMP3::Misc::descendVirtual($paramsref->{'dir'},$item,$itemnumber);
			if (SliMP3::Info::isList($item)) {
				$list_form{'descend'}     = $shortitem;
			} elsif (SliMP3::Info::isSong($item)){
				$list_form{'descend'}     = 0;
			}

			if ($filesort) {
				$list_form{'title'}		= SliMP3::Info::fileName($item);
			} else {
				$list_form{'title'}		= SliMP3::Info::standardTitle(undef,$item);
				$list_form{'artist'}    = SliMP3::Info::artist($item);
				$list_form{'album'}     = SliMP3::Info::album($item);
			}
			
			$list_form{'itempath'}  = $shortitem;
			$list_form{'odd'}	  	= ($itemnumber + $offset) % 2;
			$list_form{'player'}	= $current_player;
			
			my $anchor = anchor($list_form{'title'});
			if ($lastAnchor ne $anchor) {
				$list_form{'anchor'}  = $anchor;
				$lastAnchor = $anchor;
			}
			$list_form{'skinOverride'} = $$paramsref{'skinOverride'};
			$$paramsref{'browse_list'} .= &SliMP3::HTTP::filltemplatefile(($$paramsref{'playlist'} ? "browse_playlist_list.html" : "browse_list.html"), \%list_form);

			$itemnumber++;
		}
	} else {
		$$paramsref{'browse_list'} = "<tr><td>" . string("EMPTY") . "</td></tr>";
	}

	$output .=  &SliMP3::HTTP::filltemplatefile(($$paramsref{'playlist'} ? "browse_playlist.html" : "browse.html"), $paramsref);

	&SliMP3::HTTP::addresponse($httpclientsock, $output);
}

#
# Send the status page (what we're currently playing, contents of the playlist)
#
sub status {
	my($client, $main_form_ref, $add_playlist) = @_;
	my $current_player;
	my $songcount = 0;
	 
	if (defined($client)) {
		$current_player = $client->id;
		$songcount = SliMP3::Playlist::count($client);
		my @idparts = split(':',$current_player);
		if (SliMP3::Playlist::isSynced($client)) {
			$$main_form_ref{'player_name'} = SliMP3::Playlist::syncname($client);
		} elsif ($idparts[0] ne SliMP3::Client::name($client)) {
			$$main_form_ref{'player_name'} = SliMP3::Client::name($client);
		}
		if (SliMP3::Playlist::shuffle($client)) {
			$$main_form_ref{'shuffleon'} = "on";
		} else {
			$$main_form_ref{'shuffleoff'} = "off";
		}
	
		if (!SliMP3::Playlist::repeat($client)) {
			$$main_form_ref{'repeatoff'} = "off";
		} elsif (SliMP3::Playlist::repeat($client) == 1) {
			$$main_form_ref{'repeatone'} = "one";
		} else {
			$$main_form_ref{'repeatall'} = "all";
		}
	
		if("play" eq SliMP3::Playlist::playmode($client)) {
			$$main_form_ref{'modeplay'} = "Play";
		} elsif ("pause" eq SliMP3::Playlist::playmode($client)) {
			$$main_form_ref{'modepause'} = "Pause";
		} else {
			$$main_form_ref{'modestop'} = "Stop";
		}
	
		$$main_form_ref{'volume'} = int(SliMP3::Prefs::clientGet($client, "volume") + 0.5);
  	    $$main_form_ref{'player'} = $current_player;
	}

	if ($songcount > 0) {
		$$main_form_ref{'currentsong'}    = SliMP3::Playlist::currentSongIndex($client) + 1;
		$$main_form_ref{'thissongnum'}    = SliMP3::Playlist::currentSongIndex($client);
		$$main_form_ref{'songcount'}      = $songcount;
		$$main_form_ref{'songtitle'}      = SliMP3::Info::standardTitle(undef,SliMP3::Playlist::song($client));
		$$main_form_ref{'artist'} 	  = SliMP3::Info::artist(SliMP3::Playlist::song($client));
		$$main_form_ref{'album'} 	  = SliMP3::Info::album(SliMP3::Playlist::song($client));
		if (SliMP3::Prefs::get("playlistdir"))  { $$main_form_ref{'cansave'} = 1; };
	}
	
	my $output = "";

	if (SliMP3::Client::clientCount() > 1) {
		my %clientlist = ();
		foreach my $eachclient (SliMP3::Client::clients()) {
			next if SliMP3::Playlist::isSlave($eachclient);
			#either a master or a solo player
			$clientlist{$eachclient->id()} = SliMP3::Playlist::syncname($eachclient);
		}
		if (scalar(keys(%clientlist)) > 1) {
			$$main_form_ref{'player_chooser_list'} = options((SliMP3::Playlist::isSlave($client) ? $client->master->id : $current_player),\%clientlist,$$main_form_ref{'skinOverride'});
		}
	}
	
	if ($add_playlist) {
		$$main_form_ref{'playlist'} = playlist($client, $main_form_ref);
	}
			  
	$output .= &SliMP3::HTTP::filltemplatefile($add_playlist ? "status.html" : "status_header.html", $main_form_ref);
	return $output;
}

sub playlist {
	my $client = shift; 
	my $main_form_ref = shift;
	
	my $songcount = 0;
	if (defined($client)) {$songcount = SliMP3::Playlist::count($client);}
	my $playlist_items = '';
	if ($songcount > 0) {
		my $item;
		my %list_form;
		my ($start,$end) = pagebar($songcount,
								$$main_form_ref{'path'},
								SliMP3::Playlist::currentSongIndex($client),
								"player=" . SliMP3::HTTP::escape($client->id) . "&", 
								\$$main_form_ref{'start'}, 
								\$$main_form_ref{'playlist_header'},
								\$$main_form_ref{'playlist_pagebar'},
								$$main_form_ref{'skinOverride'});
		my $offset = $start % 2 ? 0 : 1; 
		my $webFormat = SliMP3::Prefs::getInd("titleFormat",SliMP3::Prefs::get("titleFormatWeb"));
		my $includeArtist =  ($webFormat !~ /ARTIST/);
		my $includeAlbum = ($webFormat !~ /ALBUM/) ;
		
		for( $item = $start; $item < $end + 1; $item++) {
			%list_form = ();
			$list_form{'myClientState'} = $client;
			$list_form{'num'}=$item;
			$list_form{'odd'} = ($item + $offset) % 2;
			if ($item == SliMP3::Playlist::currentSongIndex($client)) {
				$list_form{'currentsong'} = "current";
			} else {
				$list_form{'currentsong'} = undef;
			}
			my $song = SliMP3::Playlist::song($client, $item);
			$list_form{'player'}	= $$main_form_ref{'player'};
			$list_form{'title'} 	= SliMP3::Info::standardTitle(undef,$song);
			if ($includeArtist) { $list_form{'artist'} 	= SliMP3::Info::artist($song);}
			if ($includeAlbum) { $list_form{'album'} 	= SliMP3::Info::album($song);} 
			$list_form{'start'}		= $$main_form_ref{'start'};
			$list_form{'skinOverride'}		= $$main_form_ref{'skinOverride'};
			$playlist_items .= &SliMP3::HTTP::filltemplatefile("status_list.html", \%list_form);
		}
	}
	$$main_form_ref{'playlist_items'} = $playlist_items;
	return  SliMP3::HTTP::filltemplatefile("playlist.html", $main_form_ref);
}

sub search {
	my($client, %params) = @_;
	my $output = "";
	$params{'browse_list'} = " ";

	my $player = $params{'player'};
	my $query = $params{'query'};
	$params{'numresults'} = -1;

	my $descend;
	my $lastAnchor = '';
	if ($query) {
		my $otherparams = 'player=' . SliMP3::HTTP::escape($player) . 
		                  '&type=' . ($params{'type'} ? $params{'type'} : ''). 
		                  '&query=' . SliMP3::HTTP::escape($params{'query'}) . '&';
		if ($params{'type'} eq 'artist') {
			my @searchresults = SliMP3::Info::artists([], searchStringSplit($query), [], []);
			$params{'numresults'} = scalar @searchresults;
			$descend = 'true';
			my $itemnumber = 0;
			if ($params{'numresults'}) {
				my ($start,$end) = alphapagebar(\@searchresults
								,$params{'path'}
								,$otherparams
								,\$params{'start'}
								,\$params{'searchlist_pagebar'}
								,1
								,$params{'skinOverride'});
				foreach my $item ( @searchresults[$start..$end] ) {
					my %list_form=();
					$list_form{'genre'}	  = '*';
					$list_form{'artist'}  = $item;
					$list_form{'album'}	  = '';
					$list_form{'song'}	  = '';
					$list_form{'title'}   = $item;
					$list_form{'descend'} = $descend;
					$list_form{'player'} = $player;
					$list_form{'odd'}	  = ($itemnumber + 1) % 2;
					$list_form{'skinOverrride'} = $params{'skinOverride'};
					my $anchor = anchor($item, 1);
					if ($lastAnchor ne $anchor) {
						$list_form{'anchor'}  = $anchor;
						$lastAnchor = $anchor;
					}
					$itemnumber++;
					$params{'browse_list'} .= &SliMP3::HTTP::filltemplatefile("browseid3_list.html", \%list_form);
				}
			}
		} elsif ($params{'type'} eq 'album') {
			my @searchresults = SliMP3::Info::albums([], [], searchStringSplit($query), []);
			$params{'numresults'} = scalar @searchresults;
			$descend = 'true';
			my $itemnumber = 0;
			if ($params{'numresults'}) {
				my ($start,$end) = alphapagebar(\@searchresults
								,$params{'path'}
								,$otherparams
								,\$params{'start'}
								,\$params{'searchlist_pagebar'}
								,0
								,$params{'skinOverride'});
				foreach my $item ( @searchresults[$start..$end] ) {
					my %list_form=();
					$list_form{'genre'}	  = '*';
					$list_form{'artist'}  = '*';
					$list_form{'album'}	  = $item;
					$list_form{'song'}	  = '';
					$list_form{'title'}   = $item;
					$list_form{'descend'} = $descend;
					$list_form{'player'} = $player;
					$list_form{'odd'}	  = ($itemnumber + 1) % 2;
					$list_form{'skinOverrride'} = $params{'skinOverride'};
					my $anchor = anchor($item);
					if ($lastAnchor ne $anchor) {
						$list_form{'anchor'}  = $anchor;
						$lastAnchor = $anchor;
					}
					$itemnumber++;
					$params{'browse_list'} .= &SliMP3::HTTP::filltemplatefile("browseid3_list.html", \%list_form);
				}
			}
		} elsif ($params{'type'} eq 'song') {
			my @searchresults = SliMP3::Info::songs([], [], [], searchStringSplit($query));
			$params{'numresults'} = scalar @searchresults;
			my $itemnumber = 0;
			if ($params{'numresults'}) {
				my ($start,$end) = pagebar(scalar(@searchresults)
								,$params{'path'}
								,0
								,$otherparams
								,\$params{'start'}
								,\$params{'searchlist_header'}
								,\$params{'searchlist_pagebar'}
								,$params{'skinOverride'});
				foreach my $item ( @searchresults[$start..$end] ) {
					my %list_form=();
					$list_form{'genre'}	  = SliMP3::Info::genre($item);
					$list_form{'artist'}  = SliMP3::Info::artist($item);
					$list_form{'album'}	  = SliMP3::Info::album($item);
					$list_form{'itempath'} = SliMP3::Misc::safePath($item);
					$list_form{'title'}   = SliMP3::Info::standardTitle(undef, $item);
					$list_form{'descend'} = undef;
					$list_form{'player'} = $player;
					$list_form{'odd'}	  = ($itemnumber + 1) % 2;
					$list_form{'skinOverrride'} = $params{'skinOverride'};
					$itemnumber++;
					$params{'browse_list'} .= &SliMP3::HTTP::filltemplatefile("browseid3_list.html", \%list_form);
				}
			}
		}
	}

	$output .= &SliMP3::HTTP::filltemplatefile("search.html", \%params);
}

sub songinfo {
	my($client, %params) = @_;
	my $output = "";

	my $song = $params{'songurl'};
	if ($song) {
		$song = SliMP3::Misc::virtualToAbsolute($song);
		$song = SliMP3::Misc::fixPath($song);
	}

	if (!$song) {
		my $song = SliMP3::Info::songPath($params{'genre'}, $params{'artist'}, $params{'album'}, $params{'track'});
	}

	if ($song) {
		$params{'genre'} = SliMP3::Info::genre($song);
		$params{'artist'} = SliMP3::Info::artist($song);
		$params{'composer'} = SliMP3::Info::composer($song);
		$params{'album'} = SliMP3::Info::album($song);
		$params{'title'} = SliMP3::Info::title($song);
		if (SliMP3::Info::fileLength($song)) { $params{'filelength'} = SliMP3::Misc::delimitThousands(SliMP3::Info::fileLength($song)); }
		$params{'duration'} = SliMP3::Info::duration($song);
		$params{'track'} = SliMP3::Info::trackNumber($song);
		$params{'year'} = SliMP3::Info::year($song);
		$params{'tagversion'} = SliMP3::Info::tagVersion($song);

		# make urls in comments into links
		$params{'comment'} = SliMP3::Info::comment($song);
		my $comment = SliMP3::Info::comment($song);
		if ($comment) {
			if (!($comment =~ s!\b(http://[\-~A-Za-z0-9_/\.]+)!<a href=\"$1\" target=\"_blank\">$1</a>!igo)) {
				# handle emusic-type urls which don't have http://
				$comment =~ s!\b(www\.[\-~A-Za-z0-9_/\.]+)!<a href=\"http://$1\" target=\"_blank\">$1</a>!igo;
			}
		}
		$params{'comment'} = $comment;

		$params{'bitrate'} = SliMP3::Info::bitrate($song);

		my $url;
		if (SliMP3::Info::isURL($song)) {
			$url = $song;
		} else {
			my @path  = splitdir($song);
			$url = 'file://';
			foreach my $item (@path) {
				$url .= '/' . SliMP3::HTTP::escape($item);
			}
		}
		$params{'url'} = $url;
		
		$params{'itempath'} = SliMP3::Misc::safePath($song);
	}

		$output .= &SliMP3::HTTP::filltemplatefile("songinfo.html", \%params);
}

sub browseid3 {
	my($client, %params) = @_;
	my $output = "";
	my @items = ();

	my $song = $params{'song'};
	my $artist = $params{'artist'};
	my $album = $params{'album'};
	my $genre = $params{'genre'};
	my $player = $params{'player'};
	my $descend;
	my %list_form;
	my $itemnumber = 0;
	my $lastAnchor = '';

	# warn the user if the scanning isn't complete.
	if (SliMP3::MusicFolderScan::stillScanning() || SliMP3::iTunes::stillScanning()) {
		$params{'warn'} = 1;
	}

	if (SliMP3::iTunes::useiTunesLibrary()) {
		$params{'itunes'} = 1;
	}

	if (defined($genre) && $genre eq '*' && 
	    defined($artist) && $artist eq '*') {
		%list_form=();
		$list_form{'song'} = '';
		$list_form{'album'} = '';
		$list_form{'artist'} = '*';
		$list_form{'genre'} = '*';
		$list_form{'player'} = $player;
		$list_form{'pwditem'} = string('BROWSE_BY_ALBUM');
		$list_form{'skinOverrride'} = $params{'skinOverride'};
		$params{'pwd_list'} .= &SliMP3::HTTP::filltemplatefile("browseid3_pwdlist.html", \%list_form);
		$params{'browseby'} = string('BROWSE_BY_ALBUM');
	} elsif (defined($genre) && $genre eq '*') {
		%list_form=();
		$list_form{'song'} = '';
		$list_form{'artist'} = '';
		$list_form{'album'} = '';
		$list_form{'genre'} = '*';
		$list_form{'player'} = $player;
		$list_form{'pwditem'} = string('BROWSE_BY_ARTIST');
		$list_form{'skinOverrride'} = $params{'skinOverride'};
		$params{'pwd_list'} .= &SliMP3::HTTP::filltemplatefile("browseid3_pwdlist.html", \%list_form);
		$params{'browseby'} = string('BROWSE_BY_ARTIST');
	} else {
		%list_form=();
		$list_form{'song'} = '';
		$list_form{'artist'} = '';
		$list_form{'album'} = '';
		$list_form{'genre'} = '';
		$list_form{'player'} = $player;
		$list_form{'pwditem'} = string('BROWSE_BY_GENRE');
		$list_form{'skinOverrride'} = $params{'skinOverride'};
		$params{'pwd_list'} .= &SliMP3::HTTP::filltemplatefile("browseid3_pwdlist.html", \%list_form);
		$params{'browseby'} = string('BROWSE_BY_GENRE');
	};

	if ($genre && $genre ne '*') {
		%list_form=();
		$list_form{'song'} = '';
		$list_form{'artist'} = '';
		$list_form{'album'} = '';
		$list_form{'genre'} = $genre;
		$list_form{'player'} = $player;
		$list_form{'pwditem'} = $genre;
		$list_form{'skinOverrride'} = $params{'skinOverride'};
		$params{'pwd_list'} .= &SliMP3::HTTP::filltemplatefile("browseid3_pwdlist.html", \%list_form);
	}

	if ($artist && $artist ne '*') {
		%list_form=();
		$list_form{'song'} = '';
		$list_form{'album'} = '';
		$list_form{'artist'} = $artist;
		$list_form{'genre'} = $genre;
		$list_form{'pwditem'} = $artist;
		$list_form{'player'} = $player;
		$list_form{'skinOverrride'} = $params{'skinOverride'};
		$params{'pwd_list'} .= &SliMP3::HTTP::filltemplatefile("browseid3_pwdlist.html", \%list_form);
	}

	if ($album && $album ne '*') {
		%list_form=();
		$list_form{'song'} = '';
		$list_form{'album'} = $album;
		$list_form{'artist'} = $artist;
		$list_form{'genre'} = $genre;
		$list_form{'pwditem'} = $album;
		$list_form{'player'} = $player;
		$list_form{'skinOverrride'} = $params{'skinOverride'};
		$params{'pwd_list'} .= &SliMP3::HTTP::filltemplatefile("browseid3_pwdlist.html", \%list_form);
	}
	my $otherparams = 'player=' . SliMP3::HTTP::escape($player) . 
					  '&genre=' . SliMP3::HTTP::escape($genre) . 
					  '&artist=' . SliMP3::HTTP::escape($artist) . 
					  '&album=' . SliMP3::HTTP::escape($album) . 
					  '&song=' . SliMP3::HTTP::escape($song) . '&';
	if (!$genre) {
		# Browse by Genre
		@items = SliMP3::Info::genres([], [$artist], [$album], [$song]);
		if (scalar(@items)) {
			my ($start,$end) = alphapagebar(\@items
							,$params{'path'}
							,$otherparams
							,\$params{'start'}
							,\$params{'browselist_pagebar'}
							,0
							,$params{'skinOverride'});
			$descend = 'true';
			foreach my $item ( @items[$start..$end] ) {
				%list_form=();
				$list_form{'genre'}	  = $item;
				$list_form{'artist'}  = $artist;
				$list_form{'album'}	  = $album;
				$list_form{'song'}	  = $song;
				$list_form{'title'}   = $item;
				$list_form{'descend'} = $descend;
				$list_form{'player'} = $player;
				$list_form{'odd'}	  = ($itemnumber + 1) % 2;
				$list_form{'skinOverrride'} = $params{'skinOverride'};
				my $anchor = anchor($item);
				if ($lastAnchor ne $anchor) {
					$list_form{'anchor'}  = $anchor;
					$lastAnchor = $anchor;
				}
				$itemnumber++;
				$params{'browse_list'} .= &SliMP3::HTTP::filltemplatefile("browseid3_list.html", \%list_form);
			}
		}
	} elsif (!$artist) {
		# Browse by Artist
		@items = SliMP3::Info::artists([$genre], [], [$album], [$song]);
		if (scalar(@items)) {
			my ($start,$end) = alphapagebar(\@items
							,$params{'path'}
							,$otherparams
							,\$params{'start'}
							,\$params{'browselist_pagebar'}
							,1
							,$params{'skinOverride'});
			$descend = 'true';

			if (scalar(@items) > 1) {
				%list_form=();
				$list_form{'genre'}	  = $genre;
				$list_form{'artist'}  = '*';
				$list_form{'album'}	  = $album;
				$list_form{'song'}	  = $song;
				$list_form{'title'}   = string('ALL_ALBUMS');
				$list_form{'descend'} = $descend;
				$list_form{'player'} = $player;
				$list_form{'odd'}	  = ($itemnumber + 1) % 2;
				$list_form{'skinOverrride'} = $params{'skinOverride'};
				$itemnumber++;
				$params{'browse_list'} .= &SliMP3::HTTP::filltemplatefile("browseid3_list.html", \%list_form);
			}
			
			foreach my $item ( @items[$start..$end] ) {
				%list_form=();
				$list_form{'genre'}	  = $genre;
				$list_form{'artist'}  = $item;
				$list_form{'album'}	  = $album;
				$list_form{'song'}	  = $song;
				$list_form{'title'}   = $item;
				$list_form{'descend'} = $descend;
				$list_form{'player'} = $player;
				$list_form{'odd'}	  = ($itemnumber + 1) % 2;
				$list_form{'skinOverrride'} = $params{'skinOverride'};
				my $anchor = anchor($item, 1);
				if ($lastAnchor ne $anchor) {
					$list_form{'anchor'}  = $anchor;
					$lastAnchor = $anchor;
				}
				$itemnumber++;
				$params{'browse_list'} .= &SliMP3::HTTP::filltemplatefile("browseid3_list.html", \%list_form);
			}
		}
	} elsif (!$album) {
		# Browse by Album
		@items = SliMP3::Info::albums([$genre], [$artist], [], [$song]);
		if (scalar(@items)) {
			my ($start,$end) = alphapagebar(\@items
							,$params{'path'}
							,$otherparams
							,\$params{'start'}
							,\$params{'browselist_pagebar'}
							,1
							,$params{'skinOverride'});
			$descend = 'true';
			
			if (scalar(@items) > 1) {
				%list_form=();
				$list_form{'genre'}	  = $genre;
				$list_form{'artist'}  = $artist;
				$list_form{'album'}	  = '*';
				$list_form{'song'}	  = $song;
				$list_form{'title'}   = string('ALL_SONGS');
				$list_form{'descend'} = $descend;
				$list_form{'player'} = $player;
				$list_form{'odd'}	  = ($itemnumber + 1) % 2;
				$list_form{'skinOverrride'} = $params{'skinOverride'};
				$itemnumber++;
				$params{'browse_list'} .= &SliMP3::HTTP::filltemplatefile("browseid3_list.html", \%list_form);
			}
			
			foreach my $item ( @items[$start..$end] ) {
				%list_form=();
				$list_form{'genre'}	  = $genre;
				$list_form{'artist'}  = $artist;
				$list_form{'album'}	  = $item;
				$list_form{'song'}	  = $song;
				$list_form{'title'}   = $item;
				$list_form{'descend'} = $descend;
				$list_form{'player'} = $player;
				$list_form{'odd'}	  = ($itemnumber + 1) % 2;
				$list_form{'skinOverrride'} = $params{'skinOverride'};
				my $anchor = anchor($item);
				if ($lastAnchor ne $anchor) {
					$list_form{'anchor'}  = $anchor;
					$lastAnchor = $anchor;
				}
				$itemnumber++;
				$params{'browse_list'} .= &SliMP3::HTTP::filltemplatefile("browseid3_list.html", \%list_form);
			}
		}
	} else {
		# Browse by ?
		@items = SliMP3::Info::songs([$genre], [$artist], [$album], []);
		if (scalar(@items)) {
			my ($start,$end) = pagebar(scalar(@items)
							,$params{'path'}
							,0
							,$otherparams
							,\$params{'start'}
							,\$params{'browselist_header'}
							,\$params{'browselist_pagebar'}
							,$params{'skinOverride'});
			$descend = undef;
			foreach my $item ( @items[$start..$end] ) {
				%list_form=();
				my $title = SliMP3::Info::standardTitle(undef, $item);
				$list_form{'genre'}	  = SliMP3::Info::genre($item);
				$list_form{'artist'}  = SliMP3::Info::artist($item);
				$list_form{'album'}	  = SliMP3::Info::album($item);
				$list_form{'itempath'} = SliMP3::Misc::safePath($item);
				$list_form{'title'}   = $title;
				$list_form{'descend'} = $descend;
				$list_form{'player'} = $player;
				$list_form{'odd'}	  = ($itemnumber + 1) % 2;
				$list_form{'skinOverrride'} = $params{'skinOverride'};
				$itemnumber++;
				$params{'browse_list'} .= &SliMP3::HTTP::filltemplatefile("browseid3_list.html", \%list_form);
			}
		}
	}
	$params{'descend'} = $descend;

	$output .= &SliMP3::HTTP::filltemplatefile("browseid3.html", \%params);
}

sub searchStringSplit {
	my $search_string = shift;
	my @strings = ();
	foreach my $ss (split(' ',$search_string)) {
		push @strings, "*" . $ss . "*";
	}
	return \@strings;
}

sub anchor {
	my $item = shift;
	my $suppressArticles = shift;
	
	if ($suppressArticles) {
		my $articles =  SliMP3::Prefs::get("ignoredarticles");
		$articles =~ s/\s+/|/g;
		$item =~ s/^($articles)\s+//i;
	}

	return uc(substr($item, 0, 1));
}

sub options {
	#pass in the selected value and a hash of value => text pairs to get the option list filled
	#with the correct option selected.
	my ($selected,$optionref,$skinOverride) = @_;
	my $optionlist = '';
	foreach my $curroption (sort keys %{$optionref}) {
		$optionlist .= SliMP3::HTTP::filltemplatefile("select_option.html",{'selected' => ($curroption eq $selected)
											, 'key' => $curroption
											, 'value' => $optionref->{$curroption}
											, 'skinOverride' => $skinOverride});
	}
	return $optionlist;
}


#
# Build a bar of links to multiple pages of items
#
sub pagebar {
	my $itemcount = shift;
	my $path = shift;
	my $currentitem = shift;
	my $otherparams = shift;
	my $startref = shift; #will be modified
	my $headerref = shift; #will be modified
	my $pagebarref = shift; #will be modified
	my $skinOverride = shift;

	my $count = SliMP3::Prefs::get('itemsPerPage');
	my $start = (defined($$startref) && $$startref ne '') ? $$startref : (int($currentitem/$count)*$count);
	if ($start >= $itemcount) { $start = $itemcount - $count; }
	$$startref = $start;
	my $end = $start+$count-1;
	if ($end >= $itemcount) { $end = $itemcount - 1;}
	if ($itemcount > $count) {
		$$headerref = &SliMP3::HTTP::filltemplatefile("pagebarheader.html", { "start" => ($start+1), "end" => ($end+1), "itemcount" => $itemcount, 'skinOverride' => $skinOverride});

		my %pagebar = ();

		my $numpages = ceil($itemcount/$count);
		my $curpage = int($start/$count);
		my $pagesperbar = 10; #make this a preference
		my $pagebarstart = (($curpage - int($pagesperbar/2)) < 0 || $numpages <= $pagesperbar) ? 0 : ($curpage - int($pagesperbar/2));
		my $pagebarend = ($pagebarstart + $pagesperbar) > $numpages ? $numpages : ($pagebarstart + $pagesperbar);

		$pagebar{'pagesstart'} = ($pagebarstart > 0);

		if ($pagebar{'pagesstart'}) {
			$pagebar{'pagesprev'} = ($curpage - $pagesperbar) * $count;
			if ($pagebar{'pagesprev'} < 0) { $pagebar{'pagesprev'} = 0; };
		}

		if ($pagebarend < $numpages) {
			$pagebar{'pagesend'} = ($numpages -1) * $count;
			$pagebar{'pagesnext'} = ($curpage + $pagesperbar) * $count;
			if ($pagebar{'pagesnext'} > $pagebar{'pagesend'}) { $pagebar{'pagesnext'} = $pagebar{'pagesend'}; }
		}

		$pagebar{'pageprev'} = $curpage > 0 ? (($curpage - 1) * $count) : undef;
		$pagebar{'pagenext'} = ($curpage < ($numpages - 1)) ? (($curpage + 1) * $count) : undef;
		$pagebar{'otherparams'} = defined($otherparams) ? $otherparams : '';
		$pagebar{'skinOverride'} = $skinOverride;
		$pagebar{'path'} = $path;

		for (my $j = $pagebarstart;$j < $pagebarend;$j++) {
			$pagebar{'pageslist'} .= SliMP3::HTTP::filltemplatefile('pagebarlist.html'
							,{'currpage' => ($j == $curpage)
							,'itemnum0' => ($j * $count)
							,'itemnum1' => (($j * $count) + 1)
							,'pagenum' => ($j + 1)
							,'otherparams' => $otherparams
							,'skinOverride' => $skinOverride
							,'path' => $path});
		}
		$$pagebarref = SliMP3::HTTP::filltemplatefile("pagebar.html", \%pagebar);
	}
	return ($start,$end);
}

sub alphapagebar {
	my $itemsref = shift;
	my $path = shift;
	my $otherparams = shift;
	my $startref = shift; #will be modified
	my $pagebarref = shift; #will be modified
	my $ignorearticles = shift;
	my $skinOverride = shift;

	my $itemcount = scalar(@$itemsref);

	my $maxcount = SliMP3::Prefs::get('itemsPerPage');
	
	my $start = $$startref;

	if (!$start) { 
		$start = 0;
	}
	
	if ($start >= $itemcount) { 
		$start = $itemcount - $maxcount; 
	}
	
	$$startref = $start;
	
	my $end = $itemcount-1;

	if ($itemcount > $maxcount/2) {

		my %pagebar_params = ();
		if (!defined($otherparams)) {
			$otherparams = '';
		}
		$pagebar_params{'otherparams'} =  $otherparams;
		
		my $lastLetter = '';
		my $lastLetterIndex = 0;
		my $pageslist = '';
		$end = -1;
		for (my $j = 0; $j < $itemcount; $j++) {

			my $curLetter = anchor($$itemsref[$j], $ignorearticles);

			if ($lastLetter ne $curLetter) {

				if (($j - $lastLetterIndex) > $maxcount) {
					if ($end == -1 && $j > $start) {
						$end = $j - 1;
					}
					$lastLetterIndex = $j;
				}
				$pageslist .= SliMP3::HTTP::filltemplatefile('alphapagebarlist.html'
								,{'currpage' => ($lastLetterIndex == $start)
								,'itemnum0' => $lastLetterIndex
								,'itemnum1' => ($lastLetterIndex + 1)
								,'pagenum' => $curLetter
								,'fragment' => ("#" . $curLetter)
								,'otherparams' => $otherparams
								,'skinOverride' => $skinOverride
								,'path' => $path});
				
				$lastLetter = $curLetter;
			}
		}
		
		if ($end == -1) {
			$end = $itemcount - 1;
		}

		$pagebar_params{'pageslist'} = $pageslist;
		$pagebar_params{'skinOverride'} = $skinOverride;
		$$pagebarref = SliMP3::HTTP::filltemplatefile("pagebar.html", \%pagebar_params);
	}
	
	return ($start,$end);
}
