package SliMP3::RemoteStream;

# SliMP3 Server Copyright (C) 2001 Sean Adams, Slim Devices Inc.
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License, 
# version 2.

use strict;
use File::Spec::Functions qw(:ALL);
use FileHandle;
use IO::Socket qw(:DEFAULT :crlf);

use SliMP3::Display;
use SliMP3::Misc;

# Big TODO: don't block!

#$::d_remotestream = 1;

# make a request to a remote shout/icecast server
# 2001-10-25 ERL Added (recursive) redirection
sub openRemoteStream {
	my $url = shift;
	my $client = shift;
	
	my ($server, $port, $path, $user, $password) = SliMP3::Misc::crackURL($url);
	
	$::d_remotestream && msg("Opening connection to $url: [$server on port $port with path $path]\n");
	
	my $sock = IO::Socket::INET->new(PeerAddr => "$server:$port",
					LocalAddr => $main::localStreamAddr,
					 Timeout  => 10);
	if (!$sock)	{
		$::d_remotestream && msg("Can't open socket to [$server:$port]: $!");
		return undef;
	}

	$sock->autoflush(1);

	# make the request
	my $request =   "GET $path HTTP/1.1" . $CRLF . 
					"Host: " . $server . ":" . $port . $CRLF .
					"User-Agent: SliMP3/" . $::VERSION . $CRLF .
					"Accept: */*" . $CRLF .
					"Icy-MetaData:1" . $CRLF;
	
	if (defined($user) && defined($password)) {
		$request .= "Authorization: Basic " . encode_base64($user . ":" . $password,'') . $CRLF;
	}

	$request .=  $CRLF;

	$::d_remotestream && msg("Request: $request");

	print $sock $request;

	my $response = <$sock>;
	
	if ($response !~ / (\d\d\d)/) {
		close $sock;
		$::d_remotestream && msg("Invalid response code ($response) from remote stream $url\n");
		return undef; 	
	} 
	
	$response = $1;
	
	if ($response < 200) {
		$::d_remotestream && msg("Invalid response code ($response) from remote stream $url\n");
		close $sock;
		return undef;
	}

	if ($response > 399) {
		$::d_remotestream && msg("Invalid response code ($response) from remote stream $url\n");
		close $sock;
		return undef;
	}

	my $redir = '';
	while(<$sock>) {
		$::d_remotestream && msg("header: " . $_);
		if (/^icy-name:(.+)/) {
		}

		if (/^icy-metaint:(.+)/) {
			if ($client) {
				$client->shoutMetaInterval($1);
			}
		}
		
		if (/^Location:\s+(.*)/i) {
			$redir = $1;
		}

		if (/^Content-Type:\s+(.*)\015\012/i) {
			my $contenttype = $1;
			SliMP3::Info::setContentType($url,$contenttype);
		}
		
		if ($_ eq $CRLF) { 
			$::d_remotestream && msg("Recieved final blank line...\n");
			last; 
		}
	}

	if ($redir) {
		# Redirect -- maybe recursively?
		$sock->close;

		$::d_remotestream && msg("Redirect to: $redir\n");
		
		my $oldtitle = SliMP3::Info::title($url);
		
		$sock = openRemoteStream($redir, $client);
		
		# if we've opened the redirect, re-use the old title and new content type.
		
		if (defined($sock)) {
			
			if (defined($oldtitle)) {
				$::d_remotestream && msg("Saving old title: $oldtitle for $redir\n");
				SliMP3::Info::addTitleToCache($redir, $oldtitle);
			}
			
			SliMP3::Info::setContentType($url,SliMP3::Info::contentType($redir));
		}
		
		return $sock;
	}

	$::d_remotestream && msg("opened stream!\n");

	return $sock;
	
}

#
# read instream metadata for shoutcast/icecast
#
sub readMetaData {
	my $client = shift;

	my $metadataSize = 0;
	$client->mp3filehandle()->read($metadataSize, 1);
	$metadataSize = ord($metadataSize) * 16;
		
	if ($metadataSize > 0) {
		my $metadata;
		
		$client->mp3filehandle()->read($metadata, $metadataSize);

		if ($metadata =~ (/StreamTitle=\'(.*?)\'(;|$)/)) {
			my $url = SliMP3::Playlist::song($client);
			my $oldtitle = SliMP3::Info::title($url);
			my $title = $1;

			# capitalize titles that are all lowercase
			if (lc($title) eq $title) {
				$title =~ s/ (
                        			(^\w)    #at the beginning of the line
						|        # or
						(\s\w)   #preceded by whitespace
						|        # or
						(-\w)   #preceded by dash
						)
					/\U$1/xg;
			}
			
			if (defined($title) && $title ne '' && $oldtitle ne $title) {
				SliMP3::Info::addTitleToCache($url, $title);
				foreach my $everybuddy ( $client, SliMP3::Playlist::syncedWith($client)) {
					SliMP3::Animation::killAnimation($everybuddy);
				}
			}
			
			$::d_remotestream && msg("shoutcast title = $1\n");
		}

		# new song, so reset counters
		$client->songpos(0);
		$client->htmlstatusvalid(0);
		
	}
}

sub encode_base64 ($;$)
{
    my $res = "";
    my $eol = $_[1];
    $eol = "\n" unless defined $eol;
    pos($_[0]) = 0;                          # ensure start at the beginning

    $res = join '', map( pack('u',$_)=~ /^.(\S*)/, ($_[0]=~/(.{1,45})/gs));

    $res =~ tr|` -_|AA-Za-z0-9+/|;               # `# help emacs
    # fix padding at the end
    my $padding = (3 - length($_[0]) % 3) % 3;
    $res =~ s/.{$padding}$/'=' x $padding/e if $padding;
    # break encoded string into lines of no more than 76 characters each
    if (length $eol) {
        $res =~ s/(.{1,76})/$1$eol/g;
    }
    return $res;
}   

1;
__END__
