package SliMP3::VFD;

# $Id: VFD.pm,v 1.52 2003/01/27 22:18:13 blackketter Exp $

# SliMP3 Server Copyright (C) 2001 Sean Adams, Slim Devices Inc.
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License, 
# version 2.

use strict;
use SliMP3::Misc;

my %vfdCommand = ();

# the display packet header
my $header = 'l                 ';

# these codes identify the operation to 
# perform on each byte of data sent.
my $vfdCodeCmd  = pack 'B8', '00000010';       
my $vfdCodeChar = pack 'B8', '00000011'; 

# vfd.pl initiliazion:  Builds %vfdCommand, an associative array containing the 
# packed codes for all the noritake VFD commands
$vfdCommand{"BR100"} 	= pack 'B8', "00000000";
$vfdCommand{"BR25"} 	= pack 'B8', "00000011";
$vfdCommand{"BR50"} 	= pack 'B8', "00000010";
$vfdCommand{"BR75"} 	= pack 'B8', "00000001";
$vfdCommand{"CFF"}		= pack 'B8', "00001100";
$vfdCommand{"CUR"}		= pack 'B8', "00001110";
$vfdCommand{"FSET8"} 	= pack 'B8', "00110000";
$vfdCommand{"HOME"} 	= pack 'B8', "00000010";
$vfdCommand{"HOME2"} 	= pack 'B8', "11000000";
$vfdCommand{"INCSC"} 	= pack 'B8', "00000110";

my @vfdBright = ( 'BR25', 'BR25', 'BR50', 'BR75', 'BR100');


#custom characters
$SliMP3::VFD::CUSTOM0 = pack('C', 0);
$SliMP3::VFD::CUSTOM1 = pack('C', 1);
$SliMP3::VFD::CUSTOM2 = pack('C', 2);
$SliMP3::VFD::CUSTOM3 = pack('C', 3);
$SliMP3::VFD::CUSTOM4 = pack('C', 4);
$SliMP3::VFD::CUSTOM5 = pack('C', 5);
$SliMP3::VFD::CUSTOM6 = pack('C', 6);
$SliMP3::VFD::CUSTOM7 = pack('C', 7);

$SliMP3::VFD::CURSORPOS = pack('C', 8);

$SliMP3::VFD::MAXBRIGHTNESS = 4;

#
# these get remapped in the iso -> vfd conversion
#
$SliMP3::VFD::NOTESYMBOL = 0x0e;
$SliMP3::VFD::RIGHTARROW = 0x0f;

#
# Given the address of the character to edit, followed by an array of eight numbers specifying
# the bitmask of the character, caches the codes needed to create the specified character.
#
sub setCustomChar {
	my($client, $charnum, @rows)=@_;
	
	die unless ($charnum >= 0); 
	die unless ($charnum <= 7); 
	die unless ((@rows) == 8); 

	$client->vfdcustomchars->[$charnum] = \@rows;
}

#
# Adjust display brightness by delta (-3 to +3)
#
sub vfdBrightness {
	my ($client,$delta) = @_;

	if (defined($delta) ) {
		if ($delta =~ /[\+\-]\d+/) {
			$client->vfdbrightness( $client->vfdbrightness() + $delta );
		} else {
			$client->vfdbrightness( $delta );
		}
		
		$client->vfdbrightness(0) if ($client->vfdbrightness() < 0);
		$client->vfdbrightness(4) if ($client->vfdbrightness() > 4);
	
		my $temp1 = $client->prevline1();
		my $temp2 = $client->prevline2();
	
		vfdUpdate($client, $temp1, $temp2, 1);
	}
	
	return $client->vfdbrightness();
}

#
# Update the client's display - just the characters that have changed
#
sub vfdUpdate {
	my $client = shift;
	my $line1  = shift; 
	my $line2  = shift;
	my $noDoubleSize = shift; #to suppress the doublesize call (in case the input lines have already been doubled)
	my @custom;
	my $cur;
	my $pos;
	
	if (!defined($line1)) { $line1 = ' ' x 40 };
	if (!defined($line2)) { $line2 = ' ' x 40 };

	# don't display carriage returns
	$line1 =~ s/\n//;
	$line2 =~ s/\n//;

	$::d_ui && msg("vfdupdate\n");
		
	if (SliMP3::Prefs::clientGet($client,'doublesize') && !$noDoubleSize) {
		($line1, $line2) = SliMP3::Display::doubleSize($client,$line1, $line2);
	}

	$client->prevline1($line1);
	$client->prevline2($line2);

	# look for a cursor position indicator
	if ($line1 =~ /$SliMP3::VFD::CURSORPOS/) {
		$pos = $-[0];
		if ($pos < 39) {
			$cur = "_CUR" . $pos;
		}
		$line1 =~ s/$SliMP3::VFD::CURSORPOS//;
	} elsif ($line2 =~ /$SliMP3::VFD::CURSORPOS/) {
		$pos = $-[0];
		if ($pos < 39) {
			$cur = "_CUR" . ($pos + 40);
		}
		$line2 =~ s/$SliMP3::VFD::CURSORPOS//;
	} else {
		$cur = "_CFF";
	}
		
	$line1 = iso8859toVFD($line1);
	$line2 = iso8859toVFD($line2);

	$line1 = substr(($line1 . (' ' x (40 - length($line1)))),0,40);
	$line2 = substr(($line2 . (' ' x (40 - length($line2)))),0,40);

	my $vfdbrightness = SliMP3::VFD::vfdBrightness($client);

	if (defined($vfdbrightness) && ($vfdbrightness == 0)) {
		$line1 = ' ' x 40;
		$line2 = ' ' x 40;
		$cur = "_CFF";
	}
	
	# iterate over the custom characters, send them if they are defined and they are used...
	for (my $i = 0; $i < 8; $i++) {
		my $customchar = pack('C', $i);
		if (defined($client->vfdcustomchars->[$i]) && 
				(($line1 =~ /$customchar/) || ($line2 =~/$customchar/))) {
			push @custom,  '_CUSTOM' . $i;
		}
	}
		
	# eliminate leading _'s in the text, since commands start with _
	$line1 =~ s/^_/\__/g;
	$line2 =~ s/^_/\__/g;

	vfdSend($client, @custom, '_INCSC','_HOME',$line1,$line2, $cur);
  
	if ($::d_display) {
		msg(  " ---------------------------------------- \n",
		      "|$line1", ' ' x (40-length($line1)), "|\n",
		      "|$line2", ' ' x (40-length($line2)), "|\n",
		      " ----------------------------------------\n");
	}
}

sub iso8859toVFD {
	my $line = shift;

	# translate iso8859-1 to vfd charset
	$line =~ tr/\x0e\x0f\x5c\x70\x7e\x7f\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xab\xac\xad\xae\xaf\xb0\xb1\xb2\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf\xd0\xd1\xd2\xd3\xd4\xd5\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7\xe8\xe9\xea\xeb\xec\xed\xee\xef\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff/\x19\x7e\x8c\xf0\x8e\x8f\x20\x98\xec\x92\xeb\x5c\x98\x8f\xde\x63\x61\x3c\xa3\x2d\x72\xb0\xdf\xb7\x32\x33\x60\xe4\xf1\x94\x2c\x31\xdf\x3e\x25\x25\x25\x3f\x81\x81\x82\x82\x80\x81\x90\x99\x45\x45\x45\x45\x49\x49\x49\x49\x44\xee\x4f\x4f\x4f\x4f\x86\x78\x30\x55\x55\x55\x8a\x59\x70\xe2\x84\x83\x84\x84\xe1\x84\x91\x99\x65\x65\x65\x65\x69\x69\x69\x69\x95\xee\x6f\x6f\x6f\x6f\xef\xfd\x88\x75\x75\x75\xf5\x79\xf0\x79/;
	
	return $line;
}
	
#
# Converts a list of human readable VFD commands/chars into data that is sent to the player
#
sub vfdSend {
	my $client      = shift;
	my $vfddata;
	my $char;
	
	my $vfdbrightness = SliMP3::VFD::vfdBrightness($client);
	# force the display out of 4 bit mode if it got there somehow, then set the brightness
	$vfddata = $vfdCodeCmd .  (pack 'B8', "00110011") . $vfdCodeCmd .  (pack 'B8', "00000000") .
			   $vfdCodeCmd .  (pack 'B8', "00110000") .
			   $vfdCodeChar . $vfdCommand{$vfdBright[$vfdbrightness]};

	foreach my $arg (@_) {
		my $line = $arg;
		next if (!defined $line);

		if ($line =~ /^_CUR(.+)/) {

			# set the cursor position to the appropriate place.
			if ($1 < 40) {
				$vfddata .= $vfdCodeCmd.(pack 'C', (0b10000000 + $1));
			} else {
				$vfddata .= $vfdCodeCmd.(pack 'C', (0b11000000 + $1 - 40))
			}
			# turn on  the cursor			
			$vfddata .= $vfdCodeCmd.pack 'B8', '00001110';

		} elsif ($line =~ /^_(.+)/ && defined $vfdCommand{$1}) {
		  	$vfddata .= $vfdCodeCmd.$vfdCommand{$1};

		} elsif ($line =~ /^_CUSTOM(\d)/) {
			my $charindex = $1;

			my $bitmap = pack ('C8', @{$client->vfdcustomchars->[$charindex]});

			$bitmap =~ s/(.)/$vfdCodeChar$1/gos;

			$vfddata .= $vfdCodeCmd . pack('C',0b01000000 + ($charindex * 8)) . $bitmap;			
		} else {
			$line =~ s/^__/_/;
			
			$line =~ s/(.)/$vfdCodeChar$1/gos;
			$vfddata .= $line;
		}
	}

	SliMP3::Protocol::sendClient($client, $header.$vfddata);
	
	my $len = length($vfddata);
	die "Odd vfddata: $vfddata" if ($len % 2);
	die "VFDData too long: $len bytes: $vfddata" if ($len > 500);
	if (0) {
		my $i=0;
		foreach $char (split(//, $vfddata)) {
			if ($i%2==0) {
				die unless ((unpack('C', $char)  == 2) || (unpack('C', $char)  == 3));
			}
			$i++;
		}
	}
}

1;
